Imports System.Xml.Serialization

<Serializable()> _
Public Class Node
    Private Const NODE_RADIUS = 7
    Private Const NODE_DIAMETER = 2 * NODE_RADIUS

    <XmlAttributeAttribute(AttributeName:="X")> _
        Public X As Integer
    <XmlAttributeAttribute(AttributeName:="Y")> _
        Public Y As Integer
    <XmlAttributeAttribute(AttributeName:="Caption")> _
        Public Caption As String

    Public Neighbors() As Node
    Public Visited As Boolean

    Public Sub New()

    End Sub

    Public Sub New(ByVal new_x As Integer, ByVal new_y As Integer, ByVal new_caption As String)
        X = new_x
        Y = new_y
        Caption = new_caption
    End Sub

    ' Draw the node's links.
    Public Sub DrawLinks(ByVal gr As Graphics)
        Dim link_num As Integer

        If Neighbors Is Nothing Then Exit Sub

        For link_num = Neighbors.GetLowerBound(0) To Neighbors.GetUpperBound(0)
            gr.DrawLine(Pens.Black, _
                X, Y, _
                Neighbors(link_num).X, Neighbors(link_num).Y)
        Next link_num
    End Sub

    ' Erase the node's interior and then draw it.
    Public Sub DrawNode(ByVal gr As Graphics)
        gr.FillEllipse(Brushes.White, _
            X - NODE_RADIUS, Y - NODE_RADIUS, _
            NODE_DIAMETER, NODE_DIAMETER)
        gr.DrawEllipse(Pens.Black, _
            X - NODE_RADIUS, Y - NODE_RADIUS, _
            NODE_DIAMETER, NODE_DIAMETER)
        gr.DrawString(Caption, _
            New Font(New FontFamily("Times New Roman"), 8, FontStyle.Bold), _
            Brushes.Black, _
            New RectangleF( _
                X - NODE_RADIUS, _
                Y - NODE_RADIUS, _
                NODE_DIAMETER, _
                NODE_DIAMETER))
    End Sub

    ' Unmark this node and its neighbors.
    Public Sub UnmarkNodes()
        Dim link_num As Integer

        ' Unmark this node.
        If Not Visited Then Exit Sub
        Visited = False

        ' Unmark the neighbors.
        If Neighbors Is Nothing Then Exit Sub

        For link_num = Neighbors.GetLowerBound(0) To Neighbors.GetUpperBound(0)
            Neighbors(link_num).UnmarkNodes()
        Next link_num
    End Sub
End Class
